%{

/*
 * A B O U T
 *
 * scan portable draughts notation
 * interactive mode to playback single games
 * (c) 2004 Peter Chiocchetti <pch(a)myzel.net>
 *
 * This is the lexer definition.
 */

/*
 * C O P Y I N G
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */


#define YY_NO_UNPUT

#include <stdio.h>
#include <string.h>
#include "y.tab.h"
#include "pdntool.h"

#define YY_USER_ACTION col+=yyleng; // flex feature
#define YY_USER_INIT col=line=1; // flex feature

/*
 * Moves are scanned in the initial state.
 * Statements, quoted strings and comment strings
 * get their own context; fen is checked formally
 * and deliverd in pieces to the parser.
 */

%}

%option nounput

%x stmt
%x qfen
%x fstr
%x qstr
%x note

RESULT	1-0|1\/2\-1\/2|0-1|\ \*
SQUARE	[1-9]|[0-6][0-9]
FPOS	K?{SQUARE}(,K?{SQUARE})*
latin1	[¡-ÿ]

%%
\[			BEGIN(stmt); return OBRACKET;

<stmt>Event		return TEVENT;
<stmt>Date		return TDATE;
<stmt>Black		return TBLACK;
<stmt>White		return TWHITE;
<stmt>Site		return TSITE;
<stmt>Result		return TRESULT;
<stmt>Round		return TROUND;
<stmt>FEN		BEGIN(qfen); return TFEN;
<stmt>GameType		return TGAMETYPE;
<stmt>[a-z0-9]+		yylval.s=strdup(yytext); return TUNKNOWN;
<stmt>\"		BEGIN(qstr); return QUOTE;

<qfen>\"		BEGIN(fstr); return QUOTE;
<fstr>[w|b]		yylval.s=strdup(yytext); return TURN;
<fstr>:b		return BPOS;
<fstr>:w		return WPOS;
<fstr>{SQUARE},?	yylval.n=strtol(yytext, NULL, 10); return MPIECE;
<fstr>k{SQUARE},?	yylval.n=strtol(yytext+1, NULL, 10); return KPIECE;
<fstr>\.		/* discard dots */
<fstr>\"		BEGIN(stmt); return QUOTE;

<qstr>[^\"\]]*		yylval.s=strdup(yytext); return QSTRING;
<qstr>\"		BEGIN(stmt); return QUOTE;
<*>]			BEGIN(INITIAL); return EBRACKET;

<*>\{			BEGIN(note); return OBRACE;
<note>[^}\"\]]*		yylval.s=strdup(yytext); return CSTRING;
<note>\}		BEGIN(INITIAL); return EBRACE;

[0-9]+\.		return CARDINAL;
{SQUARE}(-{SQUARE})+	yylval.s=strdup(yytext); return MOVE;
{SQUARE}(-\ {SQUARE})+	yylval.s=strdup(yytext); return MOVE;
{SQUARE}(x{SQUARE})+	yylval.s=strdup(yytext); return JUMP;
{SQUARE}(x\ {SQUARE})+	yylval.s=strdup(yytext); return JUMP;
[*!?]			yylval.s=strdup(yytext); return STRENGTH;
{RESULT}		BEGIN(INITIAL); yylval.s=strdup(yytext); return RESULT;

<*>[\ \f\r\t\v]*	/* eat whitespace */;
<*>[\n]			++line; col=1;
.			/* ignore the rest */
%%
