/*
 * A B O U T
 *
 * parse portable draughts notation
 * interactive mode to playback single games
 * (c) 2004 Peter Chiocchetti <pch(a)myzel.net>
 *
 * This is the general header
 */

/*
 * C O P Y I N G
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */


// defines
#define MAXJMPLEN 16		// length of jumps string
#define MAXMOVES 500		// max half moves per game
#define SQUARE 12		// max rows/cols, canadian rules
#define PIECES 30		// max nr pieces, canadian rules

// board values
#define EMPTY    0		// un_occupied dark square
#define WHITE    1
#define BLACK    2
#define MAN      4
#define KING     8
#define FREE    16		// the light squares, always _free
#define CC       3		// change colour

// game types
#define INTERNL 20
#define ENGLISH 21
#define ITALIAN 22
#define RUSSIAN 25
// played by Martin Fierz dama; italian rules on a russian board
#define MAFIERZ 32


// a whole game
typedef struct
{
  // header
  char *event;
  char *date;
  char *black;
  char *white;
  char *site;
  char *result;
  char *round;
  int gametype;
  // FEN
  int fen;			// 1 if present
  char *turn;
  int fpos[SQUARE * SQUARE / 2 + 1];
  // moves
  int nm;			// number of moves in game
  int mh[MAXMOVES + 1][9];	// move hops 0=number of hops, 1... hops
  char *mv[MAXMOVES + 1];	// single moves
  char *ms[MAXMOVES + 1];	// move strength indicators
  char *ma[MAXMOVES + 1];	// move annotations
  // intermediate boards, before move
  int b[MAXMOVES + 1][SQUARE][SQUARE];
  // number of this game in input
  int ng;
}
PDNgame;


// CB api
typedef struct
{
  int x;			// file
  int y;			// row
}
coor;

typedef struct
{
  int jumps;			// how many jumps are there in this move?
  int newpiece;			// what type of piece appears on to
  int oldpiece;			// what disappears on from
  coor from, to;		// coordinates of the squares in 8x8 notation!
  coor path[12];		// intermediate path coordinates
  coor del[12];			// squares whose pieces are deleted
  int delpiece[12];		// what is on these squares
}
CBmove;

// globals
char *progname;
unsigned int col, line;

// protos
void gotgame (PDNgame *);
int islegal_english (int b[12][12], int, int[], CBmove *);
